# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

EXTRACT_REGION_FROM_IMAGE_METADATA = Metadata(
    id="ae946f4d1bb246c4c0e1a26e1d18d3d6f5ce930d.boutiques",
    name="ExtractRegionFromImage",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


ExtractRegionFromImageRegionMinMaxIndexParameters = typing.TypedDict('ExtractRegionFromImageRegionMinMaxIndexParameters', {
    "__STYX_TYPE__": typing.Literal["region_min_max_index"],
    "min_index": str,
    "max_index": str,
})


ExtractRegionFromImageRegionLabelParameters = typing.TypedDict('ExtractRegionFromImageRegionLabelParameters', {
    "__STYX_TYPE__": typing.Literal["region_label"],
    "label": str,
})


ExtractRegionFromImageRegionDomainImageParameters = typing.TypedDict('ExtractRegionFromImageRegionDomainImageParameters', {
    "__STYX_TYPE__": typing.Literal["region_domain_image"],
    "domain_image": InputPathType,
})


ExtractRegionFromImageRegionLabelWithImageParameters = typing.TypedDict('ExtractRegionFromImageRegionLabelWithImageParameters', {
    "__STYX_TYPE__": typing.Literal["region_label_with_image"],
    "label": str,
    "label_image": InputPathType,
})


ExtractRegionFromImageParameters = typing.TypedDict('ExtractRegionFromImageParameters', {
    "__STYX_TYPE__": typing.Literal["ExtractRegionFromImage"],
    "image_dimension": int,
    "input_image": InputPathType,
    "output_image": str,
    "region_specification": typing.Union[ExtractRegionFromImageRegionMinMaxIndexParameters, ExtractRegionFromImageRegionLabelParameters, ExtractRegionFromImageRegionDomainImageParameters, ExtractRegionFromImageRegionLabelWithImageParameters],
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "ExtractRegionFromImage": extract_region_from_image_cargs,
        "region_min_max_index": extract_region_from_image_region_min_max_index_cargs,
        "region_label": extract_region_from_image_region_label_cargs,
        "region_domain_image": extract_region_from_image_region_domain_image_cargs,
        "region_label_with_image": extract_region_from_image_region_label_with_image_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "ExtractRegionFromImage": extract_region_from_image_outputs,
    }.get(t)


def extract_region_from_image_region_min_max_index_params(
    min_index: str,
    max_index: str,
) -> ExtractRegionFromImageRegionMinMaxIndexParameters:
    """
    Build parameters.
    
    Args:
        min_index: Minimum index to define the starting point of the region.
        max_index: Maximum index to define the endpoint of the region.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "region_min_max_index",
        "min_index": min_index,
        "max_index": max_index,
    }
    return params


def extract_region_from_image_region_min_max_index_cargs(
    params: ExtractRegionFromImageRegionMinMaxIndexParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("min_index"))
    cargs.append(params.get("max_index"))
    return cargs


def extract_region_from_image_region_label_params(
    label: str,
) -> ExtractRegionFromImageRegionLabelParameters:
    """
    Build parameters.
    
    Args:
        label: Label value to extract the region corresponding to the specified\
            label.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "region_label",
        "label": label,
    }
    return params


def extract_region_from_image_region_label_cargs(
    params: ExtractRegionFromImageRegionLabelParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("label"))
    return cargs


def extract_region_from_image_region_domain_image_params(
    domain_image: InputPathType,
) -> ExtractRegionFromImageRegionDomainImageParameters:
    """
    Build parameters.
    
    Args:
        domain_image: Image defining the domain from which to extract the\
            region.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "region_domain_image",
        "domain_image": domain_image,
    }
    return params


def extract_region_from_image_region_domain_image_cargs(
    params: ExtractRegionFromImageRegionDomainImageParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("domain_image")))
    return cargs


def extract_region_from_image_region_label_with_image_params(
    label: str,
    label_image: InputPathType,
) -> ExtractRegionFromImageRegionLabelWithImageParameters:
    """
    Build parameters.
    
    Args:
        label: Label value used for the region extraction.
        label_image: Image containing label information.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "region_label_with_image",
        "label": label,
        "label_image": label_image,
    }
    return params


def extract_region_from_image_region_label_with_image_cargs(
    params: ExtractRegionFromImageRegionLabelWithImageParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("label"))
    cargs.append(execution.input_file(params.get("label_image")))
    cargs.append("1")
    return cargs


class ExtractRegionFromImageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `extract_region_from_image(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_file: OutputPathType
    """File containing the extracted region."""


def extract_region_from_image_params(
    image_dimension: int,
    input_image: InputPathType,
    output_image: str,
    region_specification: typing.Union[ExtractRegionFromImageRegionMinMaxIndexParameters, ExtractRegionFromImageRegionLabelParameters, ExtractRegionFromImageRegionDomainImageParameters, ExtractRegionFromImageRegionLabelWithImageParameters],
) -> ExtractRegionFromImageParameters:
    """
    Build parameters.
    
    Args:
        image_dimension: The dimensionality of the image.
        input_image: Path to the input image from which the region will be\
            extracted.
        output_image: Path to the output image where the extracted region will\
            be saved.
        region_specification: Specify the region to extract using indices,\
            label, domain image, or label with label image.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "ExtractRegionFromImage",
        "image_dimension": image_dimension,
        "input_image": input_image,
        "output_image": output_image,
        "region_specification": region_specification,
    }
    return params


def extract_region_from_image_cargs(
    params: ExtractRegionFromImageParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("ExtractRegionFromImage")
    cargs.append(str(params.get("image_dimension")))
    cargs.append(execution.input_file(params.get("input_image")))
    cargs.append(params.get("output_image"))
    cargs.extend(dyn_cargs(params.get("region_specification")["__STYXTYPE__"])(params.get("region_specification"), execution))
    return cargs


def extract_region_from_image_outputs(
    params: ExtractRegionFromImageParameters,
    execution: Execution,
) -> ExtractRegionFromImageOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ExtractRegionFromImageOutputs(
        root=execution.output_file("."),
        output_image_file=execution.output_file(params.get("output_image")),
    )
    return ret


def extract_region_from_image_execute(
    params: ExtractRegionFromImageParameters,
    execution: Execution,
) -> ExtractRegionFromImageOutputs:
    """
    ExtractRegionFromImage can be used to extract a specific region from a given
    image. The region can be specified via indices, label, or another domain image.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `ExtractRegionFromImageOutputs`).
    """
    params = execution.params(params)
    cargs = extract_region_from_image_cargs(params, execution)
    ret = extract_region_from_image_outputs(params, execution)
    execution.run(cargs)
    return ret


def extract_region_from_image(
    image_dimension: int,
    input_image: InputPathType,
    output_image: str,
    region_specification: typing.Union[ExtractRegionFromImageRegionMinMaxIndexParameters, ExtractRegionFromImageRegionLabelParameters, ExtractRegionFromImageRegionDomainImageParameters, ExtractRegionFromImageRegionLabelWithImageParameters],
    runner: Runner | None = None,
) -> ExtractRegionFromImageOutputs:
    """
    ExtractRegionFromImage can be used to extract a specific region from a given
    image. The region can be specified via indices, label, or another domain image.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: The dimensionality of the image.
        input_image: Path to the input image from which the region will be\
            extracted.
        output_image: Path to the output image where the extracted region will\
            be saved.
        region_specification: Specify the region to extract using indices,\
            label, domain image, or label with label image.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ExtractRegionFromImageOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(EXTRACT_REGION_FROM_IMAGE_METADATA)
    params = extract_region_from_image_params(
        image_dimension=image_dimension,
        input_image=input_image,
        output_image=output_image,
        region_specification=region_specification,
    )
    return extract_region_from_image_execute(params, execution)


__all__ = [
    "EXTRACT_REGION_FROM_IMAGE_METADATA",
    "ExtractRegionFromImageOutputs",
    "ExtractRegionFromImageParameters",
    "ExtractRegionFromImageRegionDomainImageParameters",
    "ExtractRegionFromImageRegionLabelParameters",
    "ExtractRegionFromImageRegionLabelWithImageParameters",
    "ExtractRegionFromImageRegionMinMaxIndexParameters",
    "extract_region_from_image",
    "extract_region_from_image_params",
    "extract_region_from_image_region_domain_image_params",
    "extract_region_from_image_region_label_params",
    "extract_region_from_image_region_label_with_image_params",
    "extract_region_from_image_region_min_max_index_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

REBASE_TENSOR_IMAGE_METADATA = Metadata(
    id="cb7dc4431e44610b96f7d5021a64651fd63ad535.boutiques",
    name="RebaseTensorImage",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


RebaseTensorImageParameters = typing.TypedDict('RebaseTensorImageParameters', {
    "__STYX_TYPE__": typing.Literal["RebaseTensorImage"],
    "dimension": int,
    "infile": InputPathType,
    "outfile": InputPathType,
    "method": typing.Literal["PHYSICAL", "LOCAL"],
    "reference": typing.NotRequired[InputPathType | None],
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "RebaseTensorImage": rebase_tensor_image_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "RebaseTensorImage": rebase_tensor_image_outputs,
    }.get(t)


class RebaseTensorImageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `rebase_tensor_image(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    rebased_image: OutputPathType
    """The rebased tensor image."""


def rebase_tensor_image_params(
    dimension: int,
    infile: InputPathType,
    outfile: InputPathType,
    method: typing.Literal["PHYSICAL", "LOCAL"],
    reference: InputPathType | None = None,
) -> RebaseTensorImageParameters:
    """
    Build parameters.
    
    Args:
        dimension: The dimensionality of the input image.
        infile: The input image file.
        outfile: The output image file.
        method: Method of rebasing the tensor image.
        reference: Reference image file (required if PHYSICAL or LOCAL method\
            is chosen).
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "RebaseTensorImage",
        "dimension": dimension,
        "infile": infile,
        "outfile": outfile,
        "method": method,
    }
    if reference is not None:
        params["reference"] = reference
    return params


def rebase_tensor_image_cargs(
    params: RebaseTensorImageParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("RebaseTensorImage")
    cargs.append(str(params.get("dimension")))
    cargs.append(execution.input_file(params.get("infile")))
    cargs.append(execution.input_file(params.get("outfile")))
    cargs.append(params.get("method"))
    if params.get("reference") is not None:
        cargs.append(execution.input_file(params.get("reference")))
    return cargs


def rebase_tensor_image_outputs(
    params: RebaseTensorImageParameters,
    execution: Execution,
) -> RebaseTensorImageOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = RebaseTensorImageOutputs(
        root=execution.output_file("."),
        rebased_image=execution.output_file(pathlib.Path(params.get("outfile")).name),
    )
    return ret


def rebase_tensor_image_execute(
    params: RebaseTensorImageParameters,
    execution: Execution,
) -> RebaseTensorImageOutputs:
    """
    Rebase Tensor Image using specified dimensionality and method.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `RebaseTensorImageOutputs`).
    """
    params = execution.params(params)
    cargs = rebase_tensor_image_cargs(params, execution)
    ret = rebase_tensor_image_outputs(params, execution)
    execution.run(cargs)
    return ret


def rebase_tensor_image(
    dimension: int,
    infile: InputPathType,
    outfile: InputPathType,
    method: typing.Literal["PHYSICAL", "LOCAL"],
    reference: InputPathType | None = None,
    runner: Runner | None = None,
) -> RebaseTensorImageOutputs:
    """
    Rebase Tensor Image using specified dimensionality and method.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        dimension: The dimensionality of the input image.
        infile: The input image file.
        outfile: The output image file.
        method: Method of rebasing the tensor image.
        reference: Reference image file (required if PHYSICAL or LOCAL method\
            is chosen).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `RebaseTensorImageOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(REBASE_TENSOR_IMAGE_METADATA)
    params = rebase_tensor_image_params(
        dimension=dimension,
        infile=infile,
        outfile=outfile,
        method=method,
        reference=reference,
    )
    return rebase_tensor_image_execute(params, execution)


__all__ = [
    "REBASE_TENSOR_IMAGE_METADATA",
    "RebaseTensorImageOutputs",
    "RebaseTensorImageParameters",
    "rebase_tensor_image",
    "rebase_tensor_image_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DENOISE_IMAGE_METADATA = Metadata(
    id="510d45985961f8069b7889eeca0f9b9d13f57a86.boutiques",
    name="DenoiseImage",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


DenoiseImageParameters = typing.TypedDict('DenoiseImageParameters', {
    "__STYX_TYPE__": typing.Literal["DenoiseImage"],
    "image_dimensionality": typing.NotRequired[typing.Literal[2, 3, 4] | None],
    "noise_model": typing.NotRequired[typing.Literal["Gaussian", "Rician"] | None],
    "shrink_factor": typing.NotRequired[int | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "patch_radius": typing.NotRequired[str | None],
    "search_radius": typing.NotRequired[str | None],
    "verbose": typing.NotRequired[typing.Literal[0, 1] | None],
    "input_image": InputPathType,
    "corrected_image_path": str,
    "noise_image_path": str,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "DenoiseImage": denoise_image_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "DenoiseImage": denoise_image_outputs,
    }.get(t)


class DenoiseImageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `denoise_image(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    corrected_image: OutputPathType
    """The noise corrected version of the input image."""
    noise_image: OutputPathType
    """Estimated noise image."""


def denoise_image_params(
    input_image: InputPathType,
    corrected_image_path: str,
    noise_image_path: str,
    image_dimensionality: typing.Literal[2, 3, 4] | None = None,
    noise_model: typing.Literal["Gaussian", "Rician"] | None = None,
    shrink_factor: int | None = None,
    mask_image: InputPathType | None = None,
    patch_radius: str | None = None,
    search_radius: str | None = None,
    verbose: typing.Literal[0, 1] | None = None,
) -> DenoiseImageParameters:
    """
    Build parameters.
    
    Args:
        input_image: -i, --input-image inputImageFilename. A scalar image is\
            expected as input for noise correction.
        corrected_image_path: The noise corrected version of the input image.
        noise_image_path: Estimated noise image.
        image_dimensionality: -d, --image-dimensionality 2/3/4. This option\
            forces the image to be treated as a specified-dimensional image. If not\
            specified, the program tries to infer the dimensionality from the input\
            image.
        noise_model: -n, --noise-model Rician/(Gaussian). Employ a Rician or\
            Gaussian noise model.
        shrink_factor: -s, --shrink-factor (1)/2/3/... Running noise correction\
            on large images can be time consuming. To lessen computation time, the\
            input image can be resampled. The shrink factor, specified as a single\
            integer, describes this resampling. Shrink factor = 1 is the default.
        mask_image: -x, --mask-image maskImageFilename. If a mask image is\
            specified, denoising is only performed in the mask region.
        patch_radius: -p, --patch-radius 1x1x1. Patch radius. Default is 1x1x1.
        search_radius: -r, --search-radius 2x2x2. Search radius. Default is\
            2x2x2.
        verbose: Verbose output.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "DenoiseImage",
        "input_image": input_image,
        "corrected_image_path": corrected_image_path,
        "noise_image_path": noise_image_path,
    }
    if image_dimensionality is not None:
        params["image_dimensionality"] = image_dimensionality
    if noise_model is not None:
        params["noise_model"] = noise_model
    if shrink_factor is not None:
        params["shrink_factor"] = shrink_factor
    if mask_image is not None:
        params["mask_image"] = mask_image
    if patch_radius is not None:
        params["patch_radius"] = patch_radius
    if search_radius is not None:
        params["search_radius"] = search_radius
    if verbose is not None:
        params["verbose"] = verbose
    return params


def denoise_image_cargs(
    params: DenoiseImageParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("DenoiseImage")
    if params.get("image_dimensionality") is not None:
        cargs.extend([
            "--image-dimensionality",
            str(params.get("image_dimensionality"))
        ])
    if params.get("noise_model") is not None:
        cargs.extend([
            "--noise-model",
            params.get("noise_model")
        ])
    if params.get("shrink_factor") is not None:
        cargs.extend([
            "--shrink-factor",
            str(params.get("shrink_factor"))
        ])
    if params.get("mask_image") is not None:
        cargs.extend([
            "--mask-image",
            execution.input_file(params.get("mask_image"))
        ])
    if params.get("patch_radius") is not None:
        cargs.extend([
            "--patch-radius",
            params.get("patch_radius")
        ])
    if params.get("search_radius") is not None:
        cargs.extend([
            "--search-radius",
            params.get("search_radius")
        ])
    if params.get("verbose") is not None:
        cargs.extend([
            "--verbose",
            str(params.get("verbose"))
        ])
    cargs.extend([
        "--input-image",
        execution.input_file(params.get("input_image"))
    ])
    cargs.append("--output")
    cargs.append("[" + params.get("corrected_image_path") + "," + params.get("noise_image_path") + "]")
    return cargs


def denoise_image_outputs(
    params: DenoiseImageParameters,
    execution: Execution,
) -> DenoiseImageOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = DenoiseImageOutputs(
        root=execution.output_file("."),
        corrected_image=execution.output_file(params.get("corrected_image_path")),
        noise_image=execution.output_file(params.get("noise_image_path")),
    )
    return ret


def denoise_image_execute(
    params: DenoiseImageParameters,
    execution: Execution,
) -> DenoiseImageOutputs:
    """
    Denoise an image using a spatially adaptive filter originally described in J. V.
    Manjon, P. Coupe, Luis Marti-Bonmati, D. L. Collins, and M. Robles. Adaptive
    Non-Local Means Denoising of MR Images With Spatially Varying Noise Levels,
    Journal of Magnetic Resonance Imaging, 31:192-203, June 2010.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `DenoiseImageOutputs`).
    """
    params = execution.params(params)
    cargs = denoise_image_cargs(params, execution)
    ret = denoise_image_outputs(params, execution)
    execution.run(cargs)
    return ret


def denoise_image(
    input_image: InputPathType,
    corrected_image_path: str,
    noise_image_path: str,
    image_dimensionality: typing.Literal[2, 3, 4] | None = None,
    noise_model: typing.Literal["Gaussian", "Rician"] | None = None,
    shrink_factor: int | None = None,
    mask_image: InputPathType | None = None,
    patch_radius: str | None = None,
    search_radius: str | None = None,
    verbose: typing.Literal[0, 1] | None = None,
    runner: Runner | None = None,
) -> DenoiseImageOutputs:
    """
    Denoise an image using a spatially adaptive filter originally described in J. V.
    Manjon, P. Coupe, Luis Marti-Bonmati, D. L. Collins, and M. Robles. Adaptive
    Non-Local Means Denoising of MR Images With Spatially Varying Noise Levels,
    Journal of Magnetic Resonance Imaging, 31:192-203, June 2010.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        input_image: -i, --input-image inputImageFilename. A scalar image is\
            expected as input for noise correction.
        corrected_image_path: The noise corrected version of the input image.
        noise_image_path: Estimated noise image.
        image_dimensionality: -d, --image-dimensionality 2/3/4. This option\
            forces the image to be treated as a specified-dimensional image. If not\
            specified, the program tries to infer the dimensionality from the input\
            image.
        noise_model: -n, --noise-model Rician/(Gaussian). Employ a Rician or\
            Gaussian noise model.
        shrink_factor: -s, --shrink-factor (1)/2/3/... Running noise correction\
            on large images can be time consuming. To lessen computation time, the\
            input image can be resampled. The shrink factor, specified as a single\
            integer, describes this resampling. Shrink factor = 1 is the default.
        mask_image: -x, --mask-image maskImageFilename. If a mask image is\
            specified, denoising is only performed in the mask region.
        patch_radius: -p, --patch-radius 1x1x1. Patch radius. Default is 1x1x1.
        search_radius: -r, --search-radius 2x2x2. Search radius. Default is\
            2x2x2.
        verbose: Verbose output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DenoiseImageOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(DENOISE_IMAGE_METADATA)
    params = denoise_image_params(
        image_dimensionality=image_dimensionality,
        noise_model=noise_model,
        shrink_factor=shrink_factor,
        mask_image=mask_image,
        patch_radius=patch_radius,
        search_radius=search_radius,
        verbose=verbose,
        input_image=input_image,
        corrected_image_path=corrected_image_path,
        noise_image_path=noise_image_path,
    )
    return denoise_image_execute(params, execution)


__all__ = [
    "DENOISE_IMAGE_METADATA",
    "DenoiseImageOutputs",
    "DenoiseImageParameters",
    "denoise_image",
    "denoise_image_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

PASTE_IMAGE_INTO_IMAGE_METADATA = Metadata(
    id="84fede77ce1571671d2df31a0ca7da2716107704.boutiques",
    name="PasteImageIntoImage",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


PasteImageIntoImageParameters = typing.TypedDict('PasteImageIntoImageParameters', {
    "__STYX_TYPE__": typing.Literal["PasteImageIntoImage"],
    "image_dimension": int,
    "input_canvas_image": InputPathType,
    "input_image": InputPathType,
    "output_image": str,
    "start_index": str,
    "background_label": typing.NotRequired[int | None],
    "paint_over_non_background_voxels": typing.NotRequired[typing.Literal[0, 1, 2] | None],
    "conflict_label": typing.NotRequired[int | None],
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "PasteImageIntoImage": paste_image_into_image_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "PasteImageIntoImage": paste_image_into_image_outputs,
    }.get(t)


class PasteImageIntoImageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `paste_image_into_image(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_file: OutputPathType
    """The final output image with the input image pasted onto the canvas."""


def paste_image_into_image_params(
    image_dimension: int,
    input_canvas_image: InputPathType,
    input_image: InputPathType,
    output_image: str,
    start_index: str,
    background_label: int | None = 0,
    paint_over_non_background_voxels: typing.Literal[0, 1, 2] | None = 0,
    conflict_label: int | None = -1,
) -> PasteImageIntoImageParameters:
    """
    Build parameters.
    
    Args:
        image_dimension: Specify the dimension of the images.
        input_canvas_image: The canvas image on which the input image will be\
            pasted.
        input_image: The image to be pasted onto the canvas.
        output_image: The resulting image after pasting.
        start_index: The starting index where the input image will be pasted on\
            the canvas.
        background_label: The label value considered as background.
        paint_over_non_background_voxels: Defines behavior when the input image\
            voxel is non-background and the corresponding canvas voxel is\
            background: 0 - leave as is, 1 - replace with input voxel value, 2 -\
            replace with conflict label.
        conflict_label: The label value used for conflicting non-background\
            voxels if 'paintOverNonBackgroundVoxels' is set to 2.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "PasteImageIntoImage",
        "image_dimension": image_dimension,
        "input_canvas_image": input_canvas_image,
        "input_image": input_image,
        "output_image": output_image,
        "start_index": start_index,
    }
    if background_label is not None:
        params["background_label"] = background_label
    if paint_over_non_background_voxels is not None:
        params["paint_over_non_background_voxels"] = paint_over_non_background_voxels
    if conflict_label is not None:
        params["conflict_label"] = conflict_label
    return params


def paste_image_into_image_cargs(
    params: PasteImageIntoImageParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("PasteImageIntoImage")
    cargs.append(str(params.get("image_dimension")))
    cargs.append(execution.input_file(params.get("input_canvas_image")))
    cargs.append(execution.input_file(params.get("input_image")))
    cargs.append(params.get("output_image"))
    cargs.append(params.get("start_index"))
    if params.get("background_label") is not None:
        cargs.append(str(params.get("background_label")))
    if params.get("paint_over_non_background_voxels") is not None:
        cargs.append(str(params.get("paint_over_non_background_voxels")))
    if params.get("conflict_label") is not None:
        cargs.append(str(params.get("conflict_label")))
    return cargs


def paste_image_into_image_outputs(
    params: PasteImageIntoImageParameters,
    execution: Execution,
) -> PasteImageIntoImageOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = PasteImageIntoImageOutputs(
        root=execution.output_file("."),
        output_image_file=execution.output_file(params.get("output_image")),
    )
    return ret


def paste_image_into_image_execute(
    params: PasteImageIntoImageParameters,
    execution: Execution,
) -> PasteImageIntoImageOutputs:
    """
    Paste the input image into the input canvas image. Depending on parameters, it
    can replace or merge existing voxel values.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `PasteImageIntoImageOutputs`).
    """
    params = execution.params(params)
    cargs = paste_image_into_image_cargs(params, execution)
    ret = paste_image_into_image_outputs(params, execution)
    execution.run(cargs)
    return ret


def paste_image_into_image(
    image_dimension: int,
    input_canvas_image: InputPathType,
    input_image: InputPathType,
    output_image: str,
    start_index: str,
    background_label: int | None = 0,
    paint_over_non_background_voxels: typing.Literal[0, 1, 2] | None = 0,
    conflict_label: int | None = -1,
    runner: Runner | None = None,
) -> PasteImageIntoImageOutputs:
    """
    Paste the input image into the input canvas image. Depending on parameters, it
    can replace or merge existing voxel values.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: Specify the dimension of the images.
        input_canvas_image: The canvas image on which the input image will be\
            pasted.
        input_image: The image to be pasted onto the canvas.
        output_image: The resulting image after pasting.
        start_index: The starting index where the input image will be pasted on\
            the canvas.
        background_label: The label value considered as background.
        paint_over_non_background_voxels: Defines behavior when the input image\
            voxel is non-background and the corresponding canvas voxel is\
            background: 0 - leave as is, 1 - replace with input voxel value, 2 -\
            replace with conflict label.
        conflict_label: The label value used for conflicting non-background\
            voxels if 'paintOverNonBackgroundVoxels' is set to 2.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `PasteImageIntoImageOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(PASTE_IMAGE_INTO_IMAGE_METADATA)
    params = paste_image_into_image_params(
        image_dimension=image_dimension,
        input_canvas_image=input_canvas_image,
        input_image=input_image,
        output_image=output_image,
        start_index=start_index,
        background_label=background_label,
        paint_over_non_background_voxels=paint_over_non_background_voxels,
        conflict_label=conflict_label,
    )
    return paste_image_into_image_execute(params, execution)


__all__ = [
    "PASTE_IMAGE_INTO_IMAGE_METADATA",
    "PasteImageIntoImageOutputs",
    "PasteImageIntoImageParameters",
    "paste_image_into_image",
    "paste_image_into_image_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTSUSE_LANDMARK_IMAGES_TO_GET_AFFINE_TRANSFORM_METADATA = Metadata(
    id="c11dd7c0d9840d40017d52a654120cd6141ec439.boutiques",
    name="ANTSUseLandmarkImagesToGetAffineTransform",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


AntsuseLandmarkImagesToGetAffineTransformParameters = typing.TypedDict('AntsuseLandmarkImagesToGetAffineTransformParameters', {
    "__STYX_TYPE__": typing.Literal["ANTSUseLandmarkImagesToGetAffineTransform"],
    "fixed_image": InputPathType,
    "moving_image": InputPathType,
    "transform_type": typing.Literal["rigid", "affine"],
    "output_affine": str,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "ANTSUseLandmarkImagesToGetAffineTransform": antsuse_landmark_images_to_get_affine_transform_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "ANTSUseLandmarkImagesToGetAffineTransform": antsuse_landmark_images_to_get_affine_transform_outputs,
    }.get(t)


class AntsuseLandmarkImagesToGetAffineTransformOutputs(typing.NamedTuple):
    """
    Output object returned when calling `antsuse_landmark_images_to_get_affine_transform(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    affine_transform_matrix: OutputPathType
    """The output is the affine transformation matrix file."""


def antsuse_landmark_images_to_get_affine_transform_params(
    fixed_image: InputPathType,
    moving_image: InputPathType,
    transform_type: typing.Literal["rigid", "affine"],
    output_affine: str,
) -> AntsuseLandmarkImagesToGetAffineTransformParameters:
    """
    Build parameters.
    
    Args:
        fixed_image: The fixed image containing labeled landmarks (N-ary\
            image).
        moving_image: The moving image containing labeled landmarks (N-ary\
            image).
        transform_type: Type of transform to compute: 'rigid' or 'affine'.
        output_affine: The output file for the affine transform matrix (e.g.,\
            OutAffine.txt).
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "ANTSUseLandmarkImagesToGetAffineTransform",
        "fixed_image": fixed_image,
        "moving_image": moving_image,
        "transform_type": transform_type,
        "output_affine": output_affine,
    }
    return params


def antsuse_landmark_images_to_get_affine_transform_cargs(
    params: AntsuseLandmarkImagesToGetAffineTransformParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("ANTSUseLandmarkImagesToGetAffineTransform")
    cargs.append(execution.input_file(params.get("fixed_image")))
    cargs.append(execution.input_file(params.get("moving_image")))
    cargs.append(params.get("transform_type"))
    cargs.append(params.get("output_affine"))
    return cargs


def antsuse_landmark_images_to_get_affine_transform_outputs(
    params: AntsuseLandmarkImagesToGetAffineTransformParameters,
    execution: Execution,
) -> AntsuseLandmarkImagesToGetAffineTransformOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsuseLandmarkImagesToGetAffineTransformOutputs(
        root=execution.output_file("."),
        affine_transform_matrix=execution.output_file(params.get("output_affine")),
    )
    return ret


def antsuse_landmark_images_to_get_affine_transform_execute(
    params: AntsuseLandmarkImagesToGetAffineTransformParameters,
    execution: Execution,
) -> AntsuseLandmarkImagesToGetAffineTransformOutputs:
    """
    This tool computes an affine transform (rigid or affine) from labeled landmark
    images. It requires input images to be N-ary, in the same physical space as the
    images you want to register, and to have the same landmark points defined within
    them. Landmarks are defined from the center of mass of the labels in the input
    images. ITK-snap can be used to generate the label images.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `AntsuseLandmarkImagesToGetAffineTransformOutputs`).
    """
    params = execution.params(params)
    cargs = antsuse_landmark_images_to_get_affine_transform_cargs(params, execution)
    ret = antsuse_landmark_images_to_get_affine_transform_outputs(params, execution)
    execution.run(cargs)
    return ret


def antsuse_landmark_images_to_get_affine_transform(
    fixed_image: InputPathType,
    moving_image: InputPathType,
    transform_type: typing.Literal["rigid", "affine"],
    output_affine: str,
    runner: Runner | None = None,
) -> AntsuseLandmarkImagesToGetAffineTransformOutputs:
    """
    This tool computes an affine transform (rigid or affine) from labeled landmark
    images. It requires input images to be N-ary, in the same physical space as the
    images you want to register, and to have the same landmark points defined within
    them. Landmarks are defined from the center of mass of the labels in the input
    images. ITK-snap can be used to generate the label images.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        fixed_image: The fixed image containing labeled landmarks (N-ary\
            image).
        moving_image: The moving image containing labeled landmarks (N-ary\
            image).
        transform_type: Type of transform to compute: 'rigid' or 'affine'.
        output_affine: The output file for the affine transform matrix (e.g.,\
            OutAffine.txt).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsuseLandmarkImagesToGetAffineTransformOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTSUSE_LANDMARK_IMAGES_TO_GET_AFFINE_TRANSFORM_METADATA)
    params = antsuse_landmark_images_to_get_affine_transform_params(
        fixed_image=fixed_image,
        moving_image=moving_image,
        transform_type=transform_type,
        output_affine=output_affine,
    )
    return antsuse_landmark_images_to_get_affine_transform_execute(params, execution)


__all__ = [
    "ANTSUSE_LANDMARK_IMAGES_TO_GET_AFFINE_TRANSFORM_METADATA",
    "AntsuseLandmarkImagesToGetAffineTransformOutputs",
    "AntsuseLandmarkImagesToGetAffineTransformParameters",
    "antsuse_landmark_images_to_get_affine_transform",
    "antsuse_landmark_images_to_get_affine_transform_params",
]
